/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of suplib
 *
 * suplib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * suplib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include "keyval/keyval.h"

static char *keyval_error[] =
{
  "no error\n",
  "out of memory\n",
  "no such keyword\n",
  "unbalanced quote or escape character\n",
  "invalid value\n",
  "value out of range\n",
  "unknown error\n",
};

/*

  PROCEDURE
  keyval_perror - output an error message associated with a @code{KeyValErr}

  DESCRIPTION
  Error messages are returned by the @code{keyval} routines via an integer
  code of type @code{KeyValErr}.  This routine acts much like the system
  @code{perror} routine.  It takes as input the output stream, the error
  code, the keyval specification passed to the keyval parsing routines, and
  the @var{where} argument returned by those routines.  It prints an
  error message to the passed output stream.

  AUTHOR
  Diab Jerius
*/
void
keyval_perror(
  FILE *fout,			/* where to print the error message */
  KeyValErr error,		/* which error to print */
  const char* spec,		/* the specification which caused the error */
  long where			/* where in the spec the error ocurred */
)
{
  const char *ptr = spec + where;
  
  if ( error > KeyValErr_MAXERR )
    error = KeyValErr_MAXERR;

  fputs(spec, fout);
  fputc('\n', fout);

  for (; spec != ptr; spec++)
    fputc((*spec == '\t') ? '\t' : '-', fout);

  fputs( "^\n", fout);
  fputs( keyval_error[error], fout );
}

