/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of suplib
 *
 * suplib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * suplib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <testlib/testlib.h>
#include <keyval/keyval.h>

/* where to stick the values */
typedef struct Data
{
  int	n_flies;
  double fly_lifetime;
  char *food;
  int toggle;
} Data;

KeyVal vals[] =
{
  KeyValStEntry( fly_lifetime, Key_Double, Data ),
  KeyValStEntry( food, Key_String, Data ),
  KeyValStEntry( n_flies, Key_Integer, Data ),
  KeyValStEntry( toggle, Key_Boolean, Data ),
};
#define NFIELDS ( sizeof(vals) / sizeof(KeyVal ) )

typedef struct
{
  char *string;
  KeyValErr err;
  long where;
  Data data;
} Test;

Test tests[] =
{
  /* string			err			where */
  { "toggle=xx",		KeyValErr_INVAL,	7,
    { 0, 0.0, NULL, 0 }
  },

  { "toggle=1",			KeyValErr_OK,		0,
    { 0, 0.0, NULL, 1 }
  },

  { "toggle=0",			KeyValErr_OK,		0,
    { 0, 0.0, NULL, 0 }
  },

  { "toggle=yes",		KeyValErr_OK,		0,
    { 0, 0.0, NULL, 1 }
  },

  { "toggle=no",		KeyValErr_OK,		0,
    { 0, 0.0, NULL, 0 }
  },

  { "fly_lifetime=33.2",	KeyValErr_OK,		0,
    { 0, 33.2, NULL, 0 }
  },

  { "fly_lifetime=33",		KeyValErr_OK,		0,
    { 0, 33.0, NULL, 0 }
  },

  { "fly_lifetime=snake",	KeyValErr_INVAL,	13,
    { 0, 0.0, NULL, 0 }
  },

  { "n_flies=1",		KeyValErr_OK,		0,
    { 1, 0.0, NULL, 0 }
  },

  { "n_flies=1.2",		KeyValErr_INVAL,	8,
    { 0, 0.0, NULL, 0 }
  },

  { "n_flies=sam",		KeyValErr_INVAL,	8,
    { 0, 0.0, NULL, 0 }
  },

  { "food='tiny little kibbles'",	KeyValErr_OK,	0,
    { 0, 0.0, "tiny little kibbles", 0 }
  },

  { "food='tiny",		KeyValErr_UNBAL,	0,
    { 0, 0.0, NULL, 0 }
  }
};

#define ntests (sizeof(tests) / sizeof(Test) )

static
char * flatten( Data data, char *buf )
{
  sprintf( buf,
	   "n_flies\t=%d\n"
	   "lifetime=\t%f\n"
	   "food\t=%s\n"
	   "toggle=\t%d\n",
	   data.n_flies,
	   data.fly_lifetime,
	   NULL == data.food ? "(null)" : data.food,
	   data.toggle );

  return buf;
}


int
main ( int argc, char *argv[] )
{
  char label[1024];
  char buf1[1024];
  char buf2[1024];
  int ii;
  Data data;
  KeyValErr err;
  long where;

#define LABEL(extra) (sprintf( label, "%s: %s", test.string, extra ), label)

  /******************************************************/
  START( "keyval" );

  for (ii = 0 ; ii < ntests ; ii++ )
  {
    Test test = tests[ii];

    memset( &data, 0, sizeof(data) );

    err = keyval_st( test.string, vals, NFIELDS, &data, &where );

    TEST( LABEL("err"), err, test.err );

    TEST( LABEL("where"), where, test.where );

    TESTSTRING( LABEL("values"), flatten(data, buf1), flatten(test.data, buf2) );

  }

  /******************************************************/
  SUMMARY();
  return FAILED( ) ? EXIT_FAILURE : EXIT_SUCCESS;
}
