/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of suplib
 *
 * suplib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * suplib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <testlib/testlib.h>

#include "lists/lists.h"


double data[] = {  0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0 };
double less[] = { -0.5, 0.5, 1.5, 2.5, 3.5, 4.5, 5.5 };
double more[] = {  0.5, 1.5, 2.5, 3.5, 4.5, 5.5, 6.5 };

#define NDATA (sizeof(data) / sizeof(double) )

static int
double_cmp( const void *key, const void *data )
{
  double d1 = *(double*)key;
  double d2 = *(double*)data;

  return
      d1 > d2 ?  1
    : d1 < d2 ? -1
    : 0;
}

int main( void )
{
  size_t i;
  const double *p, *lo_bnd, *hi_bnd;

  START( "lists/bnd_bsearch" );


  for ( i = 0 ; i < NDATA ; i++ )
  {
    char test[40];

    sprintf( test, "7 elements, element #%ld", i );
    TEST_RUN( test,
	    p = bnd_bsearch( data+i, data, NDATA, sizeof(double),
			     (const void **) &lo_bnd, (const void**) &hi_bnd, double_cmp ),
	    p == data+i && lo_bnd == data+i && hi_bnd == data+i,
	    1);
  }

  for ( i = 0 ; i < NDATA-1 ; i++ )
  {
    char test[40];

    sprintf( test, "6 elements, element #%ld", i );
    TEST_RUN( test,
	    p = bnd_bsearch( data+i, data, NDATA-1, sizeof(double),
			     (const void **) &lo_bnd, (const void**) &hi_bnd, double_cmp ),
	    p == data+i && lo_bnd == data+i && hi_bnd == data+i,
	    1);
  }

  TEST_RUN( "7 elements, less than 0",
	    p = bnd_bsearch( less+0, data, NDATA, sizeof(double),
			     (const void **) &lo_bnd, (const void **) &hi_bnd, double_cmp ),
	    p == NULL && lo_bnd == NULL && hi_bnd == data+0,
	    1);

  TEST_RUN( "7 elements, in-between 1 + 2",
	    p = bnd_bsearch( more+1, data, NDATA, sizeof(double),
			     (const void **) &lo_bnd, (const void **) &hi_bnd, double_cmp ),
	    p == NULL && lo_bnd == data+1 && hi_bnd == data+2,
	    1);

  TEST_RUN( "7 elements, in-between 5 + 6",
	    p = bnd_bsearch( more+5, data, NDATA, sizeof(double),
			     (const void **) &lo_bnd, (const void **) &hi_bnd, double_cmp ),
	    p == NULL && lo_bnd == data+5 && hi_bnd == data+6,
	    1);

  TEST_RUN( "7 elements, greater than 6",
	    p = bnd_bsearch( more+6, data, NDATA, sizeof(double),
			     (const void **) &lo_bnd, (const void **) &hi_bnd, double_cmp ),
	    p == NULL && lo_bnd == data+(NDATA-1) && hi_bnd == NULL,
	    1);


  TEST_RUN( "6 elements, less than 0",
	    p = bnd_bsearch( less+0, data, NDATA-1, sizeof(double),
			     (const void **) &lo_bnd, (const void **) &hi_bnd, double_cmp ),
	    p == NULL && lo_bnd == NULL && hi_bnd == data+0,
	    1);

  TEST_RUN( "6 elements, in-between 1 + 2",
	    p = bnd_bsearch( more+1, data, NDATA-1, sizeof(double),
			     (const void **) &lo_bnd, (const void **) &hi_bnd, double_cmp ),
	    p == NULL && lo_bnd == data+1 && hi_bnd == data+2,
	    1);

  TEST_RUN( "6 elements, inbetween 4 + 5",
	    p = bnd_bsearch( more+4, data, NDATA-1, sizeof(double),
			     (const void **) &lo_bnd, (const void **) &hi_bnd, double_cmp ),
	    p == NULL && lo_bnd == data+4 && hi_bnd == data+5,
	    1);

  TEST_RUN( "6 elements, greater than 5",
	    p = bnd_bsearch( more+5, data, NDATA-1, sizeof(double),
			     (const void **) &lo_bnd, (const void **) &hi_bnd, double_cmp ),
	    p == NULL && lo_bnd == data+(NDATA-2) && hi_bnd == NULL,
	    1);

  TEST_RUN( "3 elements, less than 0",
	    p = bnd_bsearch( less+0, data, 3, sizeof(double),
			     (const void **) &lo_bnd, (const void **) &hi_bnd, double_cmp ),
	    p == NULL && lo_bnd == NULL && hi_bnd == data,
	    1);

  TEST_RUN( "3 elements, greater than 1",
	    p = bnd_bsearch( more+1, data, 3, sizeof(double),
			     (const void **) &lo_bnd, (const void **) &hi_bnd, double_cmp ),
	    p == NULL && lo_bnd == data+1 && hi_bnd == data+2,
	    1);


  TEST_RUN( "3 elements, greater than 2",
	    p = bnd_bsearch( more+2, data, 3, sizeof(double),
			     (const void **) &lo_bnd, (const void **) &hi_bnd, double_cmp ),
	    p == NULL && lo_bnd == data+2 && hi_bnd == NULL,
	    1);

  TEST_RUN( "2 elements, less than 0",
	    p = bnd_bsearch( less+0, data, 2, sizeof(double),
			     (const void **) &lo_bnd, (const void **) &hi_bnd, double_cmp ),
	    p == NULL && lo_bnd == NULL && hi_bnd == data,
	    1);

  TEST_RUN( "2 elements, greater than 1",
	    p = bnd_bsearch( more+1, data, 2, sizeof(double),
			     (const void **) &lo_bnd, (const void **) &hi_bnd, double_cmp ),
	    p == NULL && lo_bnd == data+1 && hi_bnd == NULL,
	    1);

  TEST_RUN( "1 elements, less than 0",
	    p = bnd_bsearch( less+0, data, 1, sizeof(double),
			     (const void **) &lo_bnd, (const void **) &hi_bnd, double_cmp ),
	    p == NULL && lo_bnd == NULL && hi_bnd == data,
	    1);

  TEST_RUN( "1 elements, equal to 0",
	    p = bnd_bsearch( data+0, data, 1, sizeof(double),
			     (const void **) &lo_bnd, (const void **) &hi_bnd, double_cmp ),
	    p == data+0 && lo_bnd == data+0 && hi_bnd == data+0,
	    1);


  TEST_RUN( "1 elements, greater than 0",
	    p = bnd_bsearch( more+0, data, 1, sizeof(double),
			     (const void **) &lo_bnd, (const void **) &hi_bnd, double_cmp ),
	    p == NULL && lo_bnd == data && hi_bnd == NULL,
	    1);


  SUMMARY();

  return EXIT_SUCCESS;

}
