/*
 *
 *  The original routine upon which this is based is available
 *  here:
 *
 *  http://www.ams.sunysb.edu/~nkfr/ARCH/AL288.ZIP
 *
 *
 *  As there is no copyright or license notice, it is assumed to be freely
 *  redistributable with no restrictions.  
 *
 * */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/times.h>

#include "stats/stats.h"

static void
read_data( FILE *fh, int n, double *data )
{
  int ii;
  
  for (ii = 0 ; ii < n; ii++ )
  {
    if ( 1 != fscanf( fh, "%lf%*[ \n,]", &data[ii] ) )
    {
      fprintf( stderr, "unable to read [%d]\n", ii );
      exit(EXIT_FAILURE);
    }
  }
}

static void
print_data( const char* label, int n, double *data )
{
  int ii;

  printf( " %s sample: (Size = %4d):\n", label, n );
  for ( ii = 0 ; ii < n ; ii++ )
    printf( "%#4.0f", data[ii] );
  printf("\n");
}

#define ICW 1

#ifndef GSMIRN
#error must define GSMIRN
#endif

int main(int argc, char *argv[] )
{
  int ii, k, kind;
  int nx, ny;
  double *x, *y;

  double dstats[3];
  int *m;

  FILE * input;

  if ( argc < 2 )
  {
    fprintf( stderr, "missing filename on command line\n" );
    return EXIT_FAILURE;
  }

  input = fopen( argv[1], "r" );

  if ( NULL == input )
  {
    fprintf( stderr, "unable to open %s\n", argv[1] );
    return EXIT_FAILURE;
  }

  
  fscanf( input, "%d%*[ \n,]", &nx );
  fscanf( input, "%d%*[ \n,]", &ny );

  x = (double*) malloc( nx * sizeof(*x));
  y = (double*) malloc( ny * sizeof(*y));


  read_data( input, nx, x );
  read_data( input, ny, y );

  fclose( input );

  puts( "  Enter the name of the file containing two samples" );
  print_data( "1st", nx, x );
  print_data( "2nd", ny, y );
  putchar('\n');

  {
    int err = stcalc(ICW, nx, ny, x, y, &k, &m, dstats);
    if ( err )
    {
      fprintf( stderr, "stcalc returned %d\n", err );
      return EXIT_FAILURE;
    }
  }

  printf( " NX, NY (sample sizes)=%5d%5d\n", nx, ny );
  printf( " DSTATS (statistics) =%10.6f%10.6f%10.6f\n",
	  dstats[0], dstats[1], dstats[2] );
  printf( " K (number of categories) =%5d\n", k );
  printf( " M (array of categories totals) =" );
  for ( ii = 0; ii < k ; ii++ )
    printf( "%4d", m[ii] );
  printf("\n");

  putchar('\n');
  printf( " Test number (=KIND)  Statistics     P-value"
	  "     Time needed (seconds.hundredth)\n");

  for ( kind = 1 ; kind <= 3 ; kind++ )
  {
    double q;
    int err = GSMIRN( nx, ny, kind, m, dstats[kind-1], &q );

    if (err)
    {
      fprintf( stderr, "error running gsmirn for kind %d\n", kind );
      return EXIT_FAILURE;
    }
    printf( "----------------------------------------------------------------------------\n" );
    printf( "%8d%22.7f%21.16f\n", kind, dstats[kind-1], q );
  }

  free( x );
  free( y );
  free( m );

  return EXIT_SUCCESS;

}
