/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of suplib
 *
 * suplib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * suplib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <ctype.h>
#include <string.h>


#include "str/str.h"

/*

  PROCEDURE
  str_prune - remove leading and trailing white space from a string

  DESCRIPTION
  @code{str_prune} removes leading and trailing white space from a string.
  It truncates the string at the first trailing white space, then moves
  the string to the left, starting at the first non-white space character.
  The input string is modified.

  RETURNS
  It returns the pointer which it was passed.

  AUTHOR
  Diab Jerius

*/

char *
str_prune(
  char *str		/* the string to modify */
  )
{
  char *s, *t;

  if (str && *str)
  {
    /* find start of non-white space */
    for ( s = str; isspace(*s); s++ )
      ;
    if (!*s)
      return str;

    /* find end of white space */
    t = s + strlen(s);

    /* find last non-white space character */
    while ( --t >= s && isspace(*t) )
      ;
    /* seal off string */
    *(t+1) = '\0';

    /* slide string over */
    t = str;
    while (*s)
      *t++ = *s++;

    /* seal off string */
    *t = '\0';
  }

  return str;
}
