/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of suplib
 *
 * suplib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * suplib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stddef.h>
#include <string.h>

#include "str/str.h"

/*

  PROCEDURE
  str_tokcnt -  count the number of tokens in a string.

  DESCRIPTION
  @code{str_tokcnt} determines the number of tokens which would be
  returned in the course of calling the @code{strtok} function through
  completion.  It does not change the passed string.

  RETURNS
  it returns the number of tokens in the string.

  AUTHOR
  Diab Jerius
   
 */
size_t
str_tokcnt(
  const char *string,		/* string to parse */
  const char *delim		/* the delimiters to use */
)
{
  const char *start,
       *end = string;
  size_t ntok = 0;

  do
  {
    /* find start of token */
    start = end + strspn(end, delim);
    end = start + strcspn(start, delim);
    ntok++;
  } while (start != end);

  return ntok ? ntok - 1 : 0;
}
