/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of suplib
 *
 * suplib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * suplib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "str/str.h"

/*
   PROCEDURE
   tokqsplit - split a string into tokens

   DESCRIPTION
   @code{tokqsplit} splits a string into a series of tokens using
   @code{str_tokq}.  It fills a caller provided array with pointers
   to the tokens.  The caller should specify the maximum number of tokens to
   read.

   RETURNS
   It returns the actual number of tokens in the string, which
   may differ (either greater and lesser) than the number requested.
   Upon error, it returns @samp{-1}.

   AUTHOR
   Diab Jerius
 */

int
tokqsplit(
  char *str,		/* the string to split up */
  char *tok[],		/* the array that will receive the tokens */
  const char *delim,	/* the delimiters to split on */
  int ntok,		/* the maximum number of tokens to read */
  int split		/* the @code{strtokq} @code{split} argument */
)
{
  int n = 0;
  char *ptr, dchar;

  /* save errno */
  int errno_s = errno;
  errno = 0;

  /* grab as many tokens as requested */
  tok[n] = str_tokq(str, delim, split, 0, &ptr, &dchar);
  while(tok[n] && ++n < ntok)
    tok[n] = str_tokq(NULL, delim, split, 0, &ptr, &dchar);

  /* finish counting what's in the string */
  while( str_tokq(NULL, delim, split, 0, &ptr, &dchar) )
    n++;

  if (errno_s == EINVAL)
    n = -1;

  errno = errno_s;

  return n;
}
